@@ -31,6 +31,7 @@ from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import image_service as service
 from ironic.common import images
+from ironic.common import kickstart_utils as ks_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import utils as manager_utils
@@ -961,14 +962,14 @@ def build_kickstart_config_options(task):
     :returns: A dictionary of kickstart options to be used in the kickstart
               template.
     """
-    ks_options = {}
+    params = {}
     node = task.node
     manager_utils.add_secret_token(node, pregenerated=True)
     node.save()
-    ks_options['liveimg_url'] = node.instance_info['image_url']
-    ks_options['agent_token'] = node.driver_internal_info['agent_secret_token']
-    ks_options['heartbeat_url'] = _build_heartbeat_url(node.uuid)
-    return ks_options
+    params['liveimg_url'] = node.instance_info['image_url']
+    params['agent_token'] = node.driver_internal_info['agent_secret_token']
+    params['heartbeat_url'] = _build_heartbeat_url(node.uuid)
+    return {'ks_options': params}
 
 
 def get_volume_pxe_options(task):
@@ -1195,6 +1196,9 @@ def prepare_instance_kickstart_config(task, image_info, anaconda_boot=False):
     ks_options = build_kickstart_config_options(task)
     kickstart_template = image_info['ks_template'][1]
     ks_cfg = utils.render_template(kickstart_template, ks_options)
+    ks_config_drive = ks_utils.prepare_config_drive(task)
+    if ks_config_drive:
+        ks_cfg = ks_cfg + ks_config_drive
     utils.write_to_file(image_info['ks_cfg'][1], ks_cfg)
 
 
