@@ -15,7 +15,8 @@
 import contextlib
 import crypt
 import datetime
-from distutils.version import StrictVersion
+import functools
+import os
 import secrets
 import time
 
@@ -25,6 +26,7 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import excutils
+from oslo_utils import strutils
 from oslo_utils import timeutils
 
 from ironic.common import boot_devices
@@ -66,10 +68,22 @@ def node_set_boot_device(task, device, persistent=False):
 
     """
     task.driver.management.validate(task)
-    if task.node.provision_state != states.ADOPTING:
-        task.driver.management.set_boot_device(task,
-                                               device=device,
-                                               persistent=persistent)
+    if task.node.provision_state == states.ADOPTING:
+        return
+
+    force_persistent = task.node.driver_info.get(
+        'force_persistent_boot_device')
+    if force_persistent == 'Always':
+        persistent = True
+    elif force_persistent == 'Never':
+        persistent = False
+    elif force_persistent not in (None, 'Default'):
+        # Backward compatibility (used to be a boolean and only True mattered)
+        if strutils.bool_from_string(force_persistent, strict=False):
+            persistent = True
+
+    task.driver.management.set_boot_device(task, device=device,
+                                           persistent=persistent)
 
 
 def node_get_boot_mode(task):
@@ -240,11 +254,10 @@ def _can_skip_state_change(task, new_state):
         if new_state in (states.POWER_OFF, states.SOFT_POWER_OFF):
             _not_going_to_change()
             return True
-    else:
-        # if curr_state == states.ERROR:
-        # be optimistic and continue action
-        LOG.warning("Driver returns ERROR power state for node %s.",
-                    node.uuid)
+
+    LOG.info("Node %(node)s current power state is '%(state)s', "
+             "requested state is '%(new_state)s'.",
+             {'node': node.uuid, 'state': curr_state, 'new_state': new_state})
     return False
 
 
@@ -278,20 +291,19 @@ def node_power_action(task, new_state, timeout=None):
     # Set the target_power_state and clear any last_error, if we're
     # starting a new operation. This will expose to other processes
     # and clients that work is in progress.
-    if node['target_power_state'] != target_state:
-        node['target_power_state'] = target_state
-        node['last_error'] = None
-        driver_internal_info = node.driver_internal_info
-        driver_internal_info['last_power_state_change'] = str(
-            timeutils.utcnow().isoformat())
-        node.driver_internal_info = driver_internal_info
-        # NOTE(dtantsur): wipe token on shutting down, otherwise a reboot in
-        # fast-track (or an accidentally booted agent) will cause subsequent
-        # actions to fail.
-        if target_state in (states.POWER_OFF, states.SOFT_POWER_OFF,
-                            states.REBOOT, states.SOFT_REBOOT):
-            wipe_internal_info_on_power_off(node)
-        node.save()
+    node['target_power_state'] = target_state
+    node['last_error'] = None
+    driver_internal_info = node.driver_internal_info
+    driver_internal_info['last_power_state_change'] = str(
+        timeutils.utcnow().isoformat())
+    node.driver_internal_info = driver_internal_info
+    # NOTE(dtantsur): wipe token on shutting down, otherwise a reboot in
+    # fast-track (or an accidentally booted agent) will cause subsequent
+    # actions to fail.
+    if new_state in (states.POWER_OFF, states.SOFT_POWER_OFF,
+                     states.REBOOT, states.SOFT_REBOOT):
+        wipe_internal_info_on_power_off(node)
+    node.save()
 
     # take power action
     try:
@@ -398,18 +410,42 @@ def cleanup_cleanwait_timeout(task):
                     "check if the ramdisk responsible for the cleaning is "
                     "running on the node. Failed on step %(step)s.") %
                   {'step': task.node.clean_step})
+    logmsg = ("Cleaning for node %(node)s failed. %(error)s" %
+              {'node': task.node.uuid, 'error': last_error})
     # NOTE(rloo): this is called from the periodic task for cleanwait timeouts,
     # via the task manager's process_event(). The node has already been moved
     # to CLEANFAIL, so the error handler doesn't need to set the fail state.
-    cleaning_error_handler(task, msg=last_error, set_fail_state=False)
+    cleaning_error_handler(task, logmsg, errmsg=last_error,
+                           set_fail_state=False)
 
 
-def cleaning_error_handler(task, msg, tear_down_cleaning=True,
-                           set_fail_state=True):
-    """Put a failed node in CLEANFAIL and maintenance."""
+def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
+                           tear_down_cleaning=True, set_fail_state=True,
+                           set_maintenance=None):
+    """Put a failed node in CLEANFAIL and maintenance (if needed).
+
+    :param task: a TaskManager instance.
+    :param logmsg: Message to be logged.
+    :param errmsg: Message for the user. Optional, if not provided `logmsg` is
+        used.
+    :param traceback: Whether to log a traceback. Defaults to False.
+    :param tear_down_cleaning: Whether to clean up the PXE and DHCP files after
+        cleaning. Default to True.
+    :param set_fail_state: Whether to set node to failed state. Default to
+        True.
+    :param set_maintenance: Whether to set maintenance mode. If None,
+        maintenance mode will be set if and only if a clean step is being
+        executed on a node.
+    """
+    if set_maintenance is None:
+        set_maintenance = bool(task.node.clean_step)
+
+    errmsg = errmsg or logmsg
+    LOG.error(logmsg, exc_info=traceback)
     node = task.node
-    node.fault = faults.CLEAN_FAILURE
-    node.maintenance = True
+    if set_maintenance:
+        node.fault = faults.CLEAN_FAILURE
+        node.maintenance = True
 
     if tear_down_cleaning:
         try:
@@ -418,7 +454,7 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
             msg2 = ('Failed to tear down cleaning on node %(uuid)s, '
                     'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
             LOG.exception(msg2)
-            msg = _('%s. Also failed to tear down cleaning.') % msg
+            errmsg = _('%s. Also failed to tear down cleaning.') % errmsg
 
     if node.provision_state in (
             states.CLEANING,
@@ -439,10 +475,10 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
     # For manual cleaning, the target provision state is MANAGEABLE, whereas
     # for automated cleaning, it is AVAILABLE.
     manual_clean = node.target_provision_state == states.MANAGEABLE
-    node.last_error = msg
+    node.last_error = errmsg
     # NOTE(dtantsur): avoid overwriting existing maintenance_reason
-    if not node.maintenance_reason:
-        node.maintenance_reason = msg
+    if not node.maintenance_reason and set_maintenance:
+        node.maintenance_reason = errmsg
     node.save()
 
     if set_fail_state and node.provision_state != states.CLEANFAIL:
@@ -453,6 +489,8 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
 def wipe_internal_info_on_power_off(node):
     """Wipe information that should not survive reboot/power off."""
     driver_internal_info = node.driver_internal_info
+    # DHCP may result in a new IP next time.
+    driver_internal_info.pop('agent_url', None)
     if not is_agent_token_pregenerated(node):
         # Wipe the token if it's not pre-generated, otherwise we'll refuse to
         # generate it again for the newly booted agent.
@@ -460,6 +498,8 @@ def wipe_internal_info_on_power_off(node):
     # Wipe cached steps since they may change after reboot.
     driver_internal_info.pop('agent_cached_deploy_steps', None)
     driver_internal_info.pop('agent_cached_clean_steps', None)
+    # Remove TLS certificate since it's regenerated on each run.
+    driver_internal_info.pop('agent_verify_ca', None)
     node.driver_internal_info = driver_internal_info
 
 
@@ -471,6 +511,8 @@ def wipe_token_and_url(task):
     # Remove agent_url since it will be re-asserted
     # upon the next deployment attempt.
     info.pop('agent_url', None)
+    # Remove TLS certificate since it's regenerated on each run.
+    info.pop('agent_verify_ca', None)
     task.node.driver_internal_info = info
 
 
@@ -481,6 +523,7 @@ def wipe_deploy_internal_info(task):
     # Clear any leftover metadata about deployment.
     info = task.node.driver_internal_info
     info['deploy_steps'] = None
+    info.pop('user_deploy_steps', None)
     info.pop('agent_cached_deploy_steps', None)
     info.pop('deploy_step_index', None)
     info.pop('deployment_reboot', None)
@@ -500,6 +543,7 @@ def wipe_cleaning_internal_info(task):
     info.pop('clean_step_index', None)
     info.pop('cleaning_reboot', None)
     info.pop('cleaning_polling', None)
+    info.pop('cleaning_disable_ramdisk', None)
     info.pop('skip_current_clean_step', None)
     info.pop('steps_validated', None)
     task.node.driver_internal_info = info
@@ -555,6 +599,21 @@ def deploying_error_handler(task, logmsg, errmsg=None, traceback=False,
     task.process_event('fail')
 
 
+def fail_on_error(error_callback, msg, *error_args, **error_kwargs):
+    """A decorator for failing operation on failure."""
+    def wrapper(func):
+        @functools.wraps(func)
+        def wrapped(task, *args, **kwargs):
+            try:
+                return func(task, *args, **kwargs)
+            except Exception as exc:
+                errmsg = "%s. %s: %s" % (msg, exc.__class__.__name__, exc)
+                error_callback(task, errmsg, *error_args, **error_kwargs)
+
+        return wrapped
+    return wrapper
+
+
 @task_manager.require_exclusive_lock
 def abort_on_conductor_take_over(task):
     """Set node's state when a task was aborted due to conductor take over.
@@ -820,7 +879,7 @@ def notify_conductor_resume_operation(task, operation):
     from ironic.conductor import rpcapi
     uuid = task.node.uuid
     rpc = rpcapi.ConductorAPI()
-    topic = rpc.get_topic_for(task.node)
+    topic = rpc.get_current_topic()
     # Need to release the lock to let the conductor take it
     task.release_resources()
     getattr(rpc, method)(task.context, uuid, topic=topic)
@@ -839,7 +898,15 @@ def skip_automated_cleaning(node):
 
     :param node: the node to consider
     """
-    return not CONF.conductor.automated_clean and not node.automated_clean
+    if node.automated_clean:
+        return False
+    elif node.automated_clean is None:
+        return not CONF.conductor.automated_clean
+    else:
+        LOG.info("Automated cleaning is disabled via the API for "
+                 "node %(node)s",
+                 {'node': node.uuid})
+        return True
 
 
 def power_on_node_if_needed(task):
@@ -938,6 +1005,21 @@ def build_configdrive(node, configdrive):
                                 vendor_data=configdrive.get('vendor_data'))
 
 
+def get_configdrive_image(node):
+    """Get configdrive as an ISO image or a URL.
+
+    Converts the JSON representation into an image. URLs and raw contents
+    are returned unchanged.
+
+    :param node: an Ironic node object.
+    :returns: A gzipped and base64 encoded configdrive as a string.
+    """
+    configdrive = node.instance_info.get('configdrive')
+    if isinstance(configdrive, dict):
+        configdrive = build_configdrive(node, configdrive)
+    return configdrive
+
+
 def fast_track_able(task):
     """Checks if the operation can be a streamlined deployment sequence.
 
@@ -990,6 +1072,11 @@ def agent_is_alive(node, timeout=None):
     :param node: A node object.
     :param timeout: Heartbeat timeout, defaults to `fast_track_timeout`.
     """
+    # If no agent_url is present then we have powered down since the
+    # last agent heartbeat
+    if not node.driver_internal_info.get('agent_url'):
+        return False
+
     return value_within_timeout(
         node.driver_internal_info.get('agent_last_heartbeat'),
         timeout or CONF.deploy.fast_track_timeout)
@@ -1099,6 +1186,36 @@ def get_node_next_deploy_steps(task, skip_current_step=True):
                                 skip_current_step=skip_current_step)
 
 
+def update_next_step_index(task, step_type):
+    """Calculate the next step index and update the node.
+
+    :param task: A TaskManager object
+    :param step_type: The type of steps to process: 'clean' or 'deploy'.
+    :returns: Index of the next step.
+    """
+    info = task.node.driver_internal_info
+    save_required = False
+
+    try:
+        skip_current_step = info.pop('skip_current_%s_step' % step_type)
+    except KeyError:
+        skip_current_step = True
+    else:
+        save_required = True
+
+    field = ('cleaning_polling' if step_type == 'clean'
+             else 'deployment_polling')
+    if info.pop(field, None) is not None:
+        save_required = True
+
+    if save_required:
+        task.node.driver_internal_info = info
+        task.node.save()
+
+    return _get_node_next_steps(task, step_type,
+                                skip_current_step=skip_current_step)
+
+
 def add_secret_token(node, pregenerated=False):
     """Adds a secret token to driver_internal_info for IPA verification.
 
@@ -1113,6 +1230,8 @@ def add_secret_token(node, pregenerated=False):
     i_info['agent_secret_token'] = token
     if pregenerated:
         i_info['agent_secret_token_pregenerated'] = True
+    else:
+        i_info.pop('agent_secret_token_pregenerated', None)
     node.driver_internal_info = i_info
 
 
@@ -1137,8 +1256,8 @@ def is_agent_token_valid(node, token):
     """Validates if a supplied token is valid for the node.
 
     :param node: Node object
-    :token: A token value to validate against the driver_internal_info field
-            agent_sercret_token.
+    :param token: A token value to validate against the driver_internal_info
+                  field agent_secret_token.
     :returns: True if the supplied token matches the token recorded in the
               supplied node object.
     """
@@ -1149,17 +1268,6 @@ def is_agent_token_valid(node, token):
     return known_token == token
 
 
-def is_agent_token_supported(agent_version):
-    # NOTE(TheJulia): This is hoped that 6.x supports
-    # agent token capabilities and realistically needs to be updated
-    # once that version of IPA is out there in some shape or form.
-    # This allows us to gracefully allow older agent's that were
-    # launched via pre-generated agent_tokens, to still work
-    # and could likely be removed at some point down the road.
-    version = str(agent_version).replace('.dev', 'b', 1)
-    return StrictVersion(version) > StrictVersion('6.1.0')
-
-
 def is_agent_token_pregenerated(node):
     """Determines if the token was generated for out of band configuration.
 
@@ -1197,7 +1305,7 @@ def make_salt():
 def hash_password(password=''):
     """Hashes a supplied password.
 
-    :param value: Value to be hashed
+    :param password: password to be hashed
     """
     return crypt.crypt(password, make_salt())
 
@@ -1230,3 +1338,138 @@ def get_attached_vif(port):
     if inspection_vif:
         return (inspection_vif, 'inspecting')
     return (None, None)
+
+
+def store_agent_certificate(node, agent_verify_ca):
+    """Store certificate received from the agent and return its path."""
+    existing_verify_ca = node.driver_internal_info.get(
+        'agent_verify_ca')
+    if existing_verify_ca:
+        if os.path.exists(existing_verify_ca):
+            try:
+                with open(existing_verify_ca, 'rt') as fp:
+                    existing_text = fp.read()
+            except EnvironmentError:
+                with excutils.save_and_reraise_exception():
+                    LOG.exception('Could not read the existing TLS certificate'
+                                  ' for node %s', node.uuid)
+
+            if existing_text.strip() != agent_verify_ca.strip():
+                LOG.error('Content mismatch for agent_verify_ca for '
+                          'node %s', node.uuid)
+                raise exception.InvalidParameterValue(
+                    _('Detected change in ramdisk provided "agent_verify_ca"'))
+            else:
+                return existing_verify_ca
+        else:
+            LOG.info('Current agent_verify_ca was not found for node '
+                     '%s, assuming take over and storing', node.uuid)
+
+    fname = os.path.join(CONF.agent.certificates_path, '%s.crt' % node.uuid)
+    try:
+        # FIXME(dtantsur): it makes more sense to create this path on conductor
+        # start-up, but it requires reworking a ton of unit tests.
+        os.makedirs(CONF.agent.certificates_path, exist_ok=True)
+        with open(fname, 'wt') as fp:
+            fp.write(agent_verify_ca)
+    except EnvironmentError:
+        with excutils.save_and_reraise_exception():
+            LOG.exception('Could not save the TLS certificate for node %s',
+                          node.uuid)
+    else:
+        LOG.debug('Saved the custom certificate for node %(node)s to %(file)s',
+                  {'node': node.uuid, 'file': fname})
+        return fname
+
+
+def node_cache_bios_settings(task, node):
+    """Do caching of bios settings if supported by driver"""
+    try:
+        LOG.debug('Getting BIOS info for node %s', node.uuid)
+        task.driver.bios.cache_bios_settings(task)
+    except exception.UnsupportedDriverExtension:
+        LOG.warning('BIOS settings are not supported for node %s, '
+                    'skipping', node.uuid)
+    # TODO(zshi) remove this check when classic drivers are removed
+    except Exception:
+        msg = (_('Caching of bios settings failed on node %(node)s.')
+               % {'node': node.uuid})
+        LOG.exception(msg)
+
+
+def node_cache_vendor(task):
+    """Cache the vendor if it can be detected."""
+    properties = task.node.properties
+    if properties.get('vendor'):
+        return  # assume that vendors don't change on fly
+
+    try:
+        # We have no vendor stored, so we'll go ahead and
+        # call to store it.
+        vendor = task.driver.management.detect_vendor(task)
+        if not vendor:
+            return
+
+        # This function may be called without an exclusive lock, so get one
+        task.upgrade_lock(purpose='caching node vendor')
+    except exception.UnsupportedDriverExtension:
+        return
+    except Exception as exc:
+        LOG.warning('Unexpected exception when trying to detect vendor '
+                    'for node %(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+        return
+
+    props = task.node.properties
+    props['vendor'] = vendor
+    task.node.properties = props
+    task.node.save()
+    LOG.info("Detected vendor %(vendor)s for node %(node)s",
+             {'vendor': vendor, 'node': task.node.uuid})
+
+
+def node_cache_boot_mode(task):
+    """Cache boot_mode and secure_boot state if supported by driver.
+
+    Cache current boot_mode and secure_boot in ironic's node representation
+
+    :param task: a TaskManager instance containing the node to check.
+    """
+    # Try to retrieve boot mode and secure_boot state
+    try:
+        boot_mode = task.driver.management.get_boot_mode(task)
+    except exception.UnsupportedDriverExtension:
+        boot_mode = None
+    except Exception as exc:
+        LOG.warning('Unexpected exception when trying to detect boot_mode '
+                    'for node %(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+        return
+    try:
+        secure_boot = task.driver.management.get_secure_boot_state(task)
+    except exception.UnsupportedDriverExtension:
+        secure_boot = None
+    except Exception as exc:
+        LOG.warning('Unexpected exception when trying to detect secure_boot '
+                    'state for node %(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+        return
+
+    if (boot_mode != task.node.boot_mode
+        or secure_boot != task.node.secure_boot):
+        # Update node if current values different from node's last known info.
+        # Get exclusive lock in case we don't have one already.
+        task.upgrade_lock(purpose='caching boot_mode or secure_boot state')
+        task.node.boot_mode = boot_mode
+        task.node.secure_boot = secure_boot
+        task.node.save()
+        LOG.info("Updated boot_mode %(boot_mode)s, secure_boot %(secure_boot)s"
+                 "for node %(node)s",
+                 {'boot_mode': boot_mode, 'secure_boot': secure_boot,
+                  'node': task.node.uuid})
