@@ -99,11 +99,6 @@ _FASTTRACK_HEARTBEAT_ALLOWED = (states.DEPLOYWAIT, states.CLEANWAIT,
 FASTTRACK_HEARTBEAT_ALLOWED = frozenset(_FASTTRACK_HEARTBEAT_ALLOWED)
 
 
-def _get_client():
-    client = agent_client.AgentClient()
-    return client
-
-
 @METRICS.timer('post_clean_step_hook')
 def post_clean_step_hook(interface, step):
     """Decorator method for adding a post clean step hook.
@@ -372,7 +367,7 @@ def execute_step(task, step, step_type, client=None):
         completed async
     """
     if client is None:
-        client = _get_client()
+        client = agent_client.get_client(task)
     ports = objects.Port.list_by_node_id(
         task.context, task.node.id)
     call = getattr(client, 'execute_%s_step' % step_type)
@@ -401,8 +396,7 @@ def _step_failure_handler(task, msg, step_type, traceback=False):
 class HeartbeatMixin(object):
     """Mixin class implementing heartbeat processing."""
 
-    def __init__(self):
-        self._client = _get_client()
+    collect_deploy_logs = True
 
     def reboot_to_instance(self, task):
         """Method invoked after the deployment is completed.
@@ -498,7 +492,7 @@ class HeartbeatMixin(object):
             # Do not call the error handler is the node is already DEPLOYFAIL
             if node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT):
                 deploy_utils.set_failed_state(
-                    task, last_error, collect_logs=bool(self._client))
+                    task, last_error, collect_logs=self.collect_deploy_logs)
 
     def _heartbeat_clean_wait(self, task):
         node = task.node
@@ -506,8 +500,11 @@ class HeartbeatMixin(object):
         try:
             node.touch_provisioning()
             if not node.clean_step:
-                LOG.debug('Node %s just booted to start cleaning.',
-                          node.uuid)
+                kind = ('manual'
+                        if node.target_provision_state == states.MANAGEABLE
+                        else 'automated')
+                LOG.debug('Node %s just booted to start %s cleaning',
+                          node.uuid, kind)
                 msg = _('Node failed to start the first cleaning step')
                 # First, cache the clean steps
                 self.refresh_clean_steps(task)
@@ -576,7 +573,8 @@ class HeartbeatMixin(object):
             return
 
         node = task.node
-        LOG.debug('Heartbeat from node %s', node.uuid)
+        LOG.debug('Heartbeat from node %s in state %s (target state %s)',
+                  node.uuid, node.provision_state, node.target_provision_state)
         driver_internal_info = node.driver_internal_info
         driver_internal_info['agent_url'] = callback_url
         driver_internal_info['agent_version'] = agent_version
@@ -621,7 +619,8 @@ class HeartbeatMixin(object):
         """
         node = task.node
         try:
-            result = self._client.finalize_rescue(node)
+            client = agent_client.get_client(task)
+            result = client.finalize_rescue(node)
         except exception.IronicException as e:
             raise exception.InstanceRescueFailure(node=node.uuid,
                                                   instance=node.instance_uuid,
@@ -644,19 +643,16 @@ class HeartbeatMixin(object):
 
 
 class AgentBaseMixin(object):
-    """Mixin with base methods not relying on any deploy steps."""
+    """Mixin with base methods not relying on any deploy steps.
+
+    Provides full support for in-band and out-of-band cleaning and
+    the machinery to support both deploy and clean in-band steps.
+    """
 
     def should_manage_boot(self, task):
         """Whether agent boot is managed by ironic."""
         return True
 
-    def refresh_steps(self, task, step_type):
-        """Refresh the node's cached steps.
-
-        :param task: a TaskManager instance
-        :param step_type: "clean" or "deploy"
-        """
-
     @METRICS.timer('AgentBaseMixin.tear_down')
     @task_manager.require_exclusive_lock
     def tear_down(self, task):
@@ -706,7 +702,7 @@ class AgentBaseMixin(object):
         """
         pass
 
-    @METRICS.timer('AgentDeployMixin.prepare_cleaning')
+    @METRICS.timer('AgentBaseMixin.prepare_cleaning')
     def prepare_cleaning(self, task):
         """Boot into the agent to prepare for cleaning.
 
@@ -724,7 +720,7 @@ class AgentBaseMixin(object):
             self.refresh_steps(task, 'clean')
         return result
 
-    @METRICS.timer('AgentDeployMixin.tear_down_cleaning')
+    @METRICS.timer('AgentBaseMixin.tear_down_cleaning')
     def tear_down_cleaning(self, task):
         """Clean up the PXE and DHCP files after cleaning.
 
@@ -735,64 +731,7 @@ class AgentBaseMixin(object):
         deploy_utils.tear_down_inband_cleaning(
             task, manage_boot=self.should_manage_boot(task))
 
-
-class AgentOobStepsMixin(object):
-    """Mixin with out-of-band deploy steps."""
-
-    @METRICS.timer('AgentDeployMixin.switch_to_tenant_network')
-    @base.deploy_step(priority=30)
-    @task_manager.require_exclusive_lock
-    def switch_to_tenant_network(self, task):
-        """Deploy step to switch the node to the tenant network.
-
-        :param task: a TaskManager object containing the node
-        """
-        try:
-            with manager_utils.power_state_for_network_configuration(task):
-                task.driver.network.remove_provisioning_network(task)
-                task.driver.network.configure_tenant_networks(task)
-        except Exception as e:
-            msg = (_('Error changing node %(node)s to tenant networks after '
-                     'deploy. %(cls)s: %(error)s') %
-                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
-                    'error': e})
-            # NOTE(mgoddard): Don't collect logs since the node has been
-            # powered off.
-            log_and_raise_deployment_error(task, msg, collect_logs=False,
-                                           exc=e)
-
-    @METRICS.timer('AgentDeployMixin.boot_instance')
-    @base.deploy_step(priority=20)
-    @task_manager.require_exclusive_lock
-    def boot_instance(self, task):
-        """Deploy step to boot the final instance.
-
-        :param task: a TaskManager object containing the node
-        """
-        can_power_on = (states.POWER_ON in
-                        task.driver.power.get_supported_power_states(task))
-        try:
-            if can_power_on:
-                manager_utils.node_power_action(task, states.POWER_ON)
-            else:
-                LOG.debug('Not trying to power on node %s that does not '
-                          'support powering on, assuming already running',
-                          task.node.uuid)
-        except Exception as e:
-            msg = (_('Error booting node %(node)s after deploy. '
-                     '%(cls)s: %(error)s') %
-                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
-                    'error': e})
-            # NOTE(mgoddard): Don't collect logs since the node has been
-            # powered off.
-            log_and_raise_deployment_error(task, msg, collect_logs=False,
-                                           exc=e)
-
-
-class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
-    """Mixin with deploy methods."""
-
-    @METRICS.timer('AgentDeployMixin.get_clean_steps')
+    @METRICS.timer('AgentBaseMixin.get_clean_steps')
     def get_clean_steps(self, task):
         """Get the list of clean steps from the agent.
 
@@ -811,26 +750,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
             task, 'clean', interface='deploy',
             override_priorities=new_priorities)
 
-    @METRICS.timer('AgentDeployMixin.get_deploy_steps')
-    def get_deploy_steps(self, task):
-        """Get the list of deploy steps from the agent.
-
-        :param task: a TaskManager object containing the node
-        :raises InstanceDeployFailure: if the deploy steps are not yet
-            available (cached), for example, when a node has just been
-            enrolled and has not been deployed yet.
-        :returns: A list of deploy step dictionaries
-        """
-        steps = super(AgentDeployMixin, self).get_deploy_steps(task)[:]
-        ib_steps = get_steps(task, 'deploy', interface='deploy')
-        # NOTE(dtantsur): we allow in-band steps to be shadowed by out-of-band
-        # ones, see the docstring of execute_deploy_step for details.
-        steps += [step for step in ib_steps
-                  # FIXME(dtantsur): nested loops are not too efficient
-                  if not conductor_steps.find_step(steps, step)]
-        return steps
-
-    @METRICS.timer('AgentDeployMixin.refresh_steps')
+    @METRICS.timer('AgentBaseMixin.refresh_steps')
     def refresh_steps(self, task, step_type):
         """Refresh the node's cached clean/deploy steps from the booted agent.
 
@@ -853,7 +773,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                   {'node': node.uuid, 'type': step_type,
                    'steps': previous_steps})
 
-        call = getattr(self._client, 'get_%s_steps' % step_type)
+        client = agent_client.get_client(task)
+        call = getattr(client, 'get_%s_steps' % step_type)
         try:
             agent_result = call(node, task.ports).get('command_result', {})
         except exception.AgentInProgress as exc:
@@ -915,7 +836,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                   '%(steps)s', {'node': node.uuid, 'steps': steps,
                                 'type': step_type})
 
-    @METRICS.timer('AgentDeployMixin.execute_clean_step')
+    @METRICS.timer('AgentBaseMixin.execute_clean_step')
     def execute_clean_step(self, task, step):
         """Execute a clean step asynchronously on the agent.
 
@@ -927,37 +848,6 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         """
         return execute_step(task, step, 'clean')
 
-    @METRICS.timer('AgentDeployMixin.execute_deploy_step')
-    def execute_deploy_step(self, task, step):
-        """Execute a deploy step.
-
-        We're trying to find a step among both out-of-band and in-band steps.
-        In case of duplicates, out-of-band steps take priority. This property
-        allows having an out-of-band deploy step that calls into
-        a corresponding in-band step after some preparation (e.g. with
-        additional input).
-
-        :param task: a TaskManager object containing the node
-        :param step: a deploy step dictionary to execute
-        :raises: InstanceDeployFailure if the agent does not return a command
-            status
-        :returns: states.DEPLOYWAIT to signify the step will be completed async
-        """
-        agent_running = task.node.driver_internal_info.get(
-            'agent_cached_deploy_steps')
-        oob_steps = self.deploy_steps
-
-        if conductor_steps.find_step(oob_steps, step):
-            return super(AgentDeployMixin, self).execute_deploy_step(
-                task, step)
-        elif not agent_running:
-            raise exception.InstanceDeployFailure(
-                _('Deploy step %(step)s has not been found. Available '
-                  'out-of-band steps: %(oob)s. Agent is not running.') %
-                {'step': step, 'oob': oob_steps})
-        else:
-            return execute_step(task, step, 'deploy')
-
     def _process_version_mismatch(self, task, step_type):
         node = task.node
         # For manual clean, the target provision state is MANAGEABLE, whereas
@@ -1019,7 +909,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
 
         manager_utils.notify_conductor_resume_operation(task, step_type)
 
-    @METRICS.timer('AgentDeployMixin.process_next_step')
+    @METRICS.timer('AgentBaseMixin.process_next_step')
     def process_next_step(self, task, step_type, **kwargs):
         """Start the next clean/deploy step if the previous one is complete.
 
@@ -1039,7 +929,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         assert step_type in ('clean', 'deploy')
 
         node = task.node
-        agent_commands = self._client.get_commands_status(task.node)
+        client = agent_client.get_client(task)
+        agent_commands = client.get_commands_status(task.node)
 
         if _freshly_booted(agent_commands, step_type):
             field = ('cleaning_reboot' if step_type == 'clean'
@@ -1116,6 +1007,113 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                     'type': step_type})
             return _step_failure_handler(task, msg, step_type)
 
+
+class AgentOobStepsMixin(object):
+    """Mixin with out-of-band deploy steps."""
+
+    @METRICS.timer('AgentOobStepsMixin.switch_to_tenant_network')
+    @base.deploy_step(priority=30)
+    @task_manager.require_exclusive_lock
+    def switch_to_tenant_network(self, task):
+        """Deploy step to switch the node to the tenant network.
+
+        :param task: a TaskManager object containing the node
+        """
+        try:
+            with manager_utils.power_state_for_network_configuration(task):
+                task.driver.network.remove_provisioning_network(task)
+                task.driver.network.configure_tenant_networks(task)
+        except Exception as e:
+            msg = (_('Error changing node %(node)s to tenant networks after '
+                     'deploy. %(cls)s: %(error)s') %
+                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            # NOTE(mgoddard): Don't collect logs since the node has been
+            # powered off.
+            log_and_raise_deployment_error(task, msg, collect_logs=False,
+                                           exc=e)
+
+    @METRICS.timer('AgentOobStepsMixin.boot_instance')
+    @base.deploy_step(priority=20)
+    @task_manager.require_exclusive_lock
+    def boot_instance(self, task):
+        """Deploy step to boot the final instance.
+
+        :param task: a TaskManager object containing the node
+        """
+        can_power_on = (states.POWER_ON in
+                        task.driver.power.get_supported_power_states(task))
+        try:
+            if can_power_on:
+                manager_utils.node_power_action(task, states.POWER_ON)
+            else:
+                LOG.debug('Not trying to power on node %s that does not '
+                          'support powering on, assuming already running',
+                          task.node.uuid)
+        except Exception as e:
+            msg = (_('Error booting node %(node)s after deploy. '
+                     '%(cls)s: %(error)s') %
+                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            # NOTE(mgoddard): Don't collect logs since the node has been
+            # powered off.
+            log_and_raise_deployment_error(task, msg, collect_logs=False,
+                                           exc=e)
+
+
+class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
+    """Mixin with deploy methods."""
+
+    @METRICS.timer('AgentDeployMixin.get_deploy_steps')
+    def get_deploy_steps(self, task):
+        """Get the list of deploy steps from the agent.
+
+        :param task: a TaskManager object containing the node
+        :raises InstanceDeployFailure: if the deploy steps are not yet
+            available (cached), for example, when a node has just been
+            enrolled and has not been deployed yet.
+        :returns: A list of deploy step dictionaries
+        """
+        steps = super(AgentDeployMixin, self).get_deploy_steps(task)[:]
+        ib_steps = get_steps(task, 'deploy', interface='deploy')
+        # NOTE(dtantsur): we allow in-band steps to be shadowed by out-of-band
+        # ones, see the docstring of execute_deploy_step for details.
+        steps += [step for step in ib_steps
+                  # FIXME(dtantsur): nested loops are not too efficient
+                  if not conductor_steps.find_step(steps, step)]
+        return steps
+
+    @METRICS.timer('AgentDeployMixin.execute_deploy_step')
+    def execute_deploy_step(self, task, step):
+        """Execute a deploy step.
+
+        We're trying to find a step among both out-of-band and in-band steps.
+        In case of duplicates, out-of-band steps take priority. This property
+        allows having an out-of-band deploy step that calls into
+        a corresponding in-band step after some preparation (e.g. with
+        additional input).
+
+        :param task: a TaskManager object containing the node
+        :param step: a deploy step dictionary to execute
+        :raises: InstanceDeployFailure if the agent does not return a command
+            status
+        :returns: states.DEPLOYWAIT to signify the step will be completed async
+        """
+        agent_running = task.node.driver_internal_info.get(
+            'agent_cached_deploy_steps')
+        oob_steps = self.deploy_steps
+
+        if conductor_steps.find_step(oob_steps, step):
+            return super(AgentDeployMixin, self).execute_deploy_step(
+                task, step)
+        elif not agent_running:
+            raise exception.InstanceDeployFailure(
+                _('Deploy step %(step)s has not been found. Available '
+                  'out-of-band steps: %(oob)s. Agent is not running.') %
+                {'step': step, 'oob': oob_steps})
+        else:
+            return execute_step(task, step, 'deploy')
+
     @METRICS.timer('AgentDeployMixin.tear_down_agent')
     @base.deploy_step(priority=40)
     @task_manager.require_exclusive_lock
@@ -1148,16 +1146,17 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         can_power_on = (states.POWER_ON in
                         task.driver.power.get_supported_power_states(task))
 
+        client = agent_client.get_client(task)
         try:
             if not can_power_on:
                 LOG.info('Power interface of node %(node)s does not support '
                          'power on, using reboot to switch to the instance',
                          node.uuid)
-                self._client.sync(node)
+                client.sync(node)
                 manager_utils.node_power_action(task, states.REBOOT)
             elif not oob_power_off:
                 try:
-                    self._client.power_off(node)
+                    client.power_off(node)
                 except Exception as e:
                     LOG.warning('Failed to soft power off node %(node_uuid)s. '
                                 '%(cls)s: %(error)s',
@@ -1180,7 +1179,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 # Flush the file system prior to hard rebooting the node
-                result = self._client.sync(node)
+                result = client.sync(node)
                 error = result.get('faultstring')
                 if error:
                     if 'Unknown command' in error:
@@ -1330,7 +1329,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                       'partition %(part)s, EFI system partition %(efi)s',
                       {'node': node.uuid, 'part': root_uuid,
                        'efi': efi_system_part_uuid})
-            result = self._client.install_bootloader(
+            client = agent_client.get_client(task)
+            result = client.install_bootloader(
                 node, root_uuid=root_uuid,
                 efi_system_part_uuid=efi_system_part_uuid,
                 prep_boot_part_uuid=prep_boot_part_uuid,
