@@ -30,6 +30,7 @@ from ironic.common import faults
 from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import image_service
+from ironic.common import images
 from ironic.common import keystone
 from ironic.common import states
 from ironic.common import utils
@@ -54,7 +55,7 @@ LOG = logging.getLogger(__name__)
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 SUPPORTED_CAPABILITIES = {
-    'boot_option': ('local', 'netboot', 'ramdisk'),
+    'boot_option': ('local', 'netboot', 'ramdisk', 'kickstart'),
     'boot_mode': ('bios', 'uefi'),
     'secure_boot': ('true', 'false'),
     'trusted_boot': ('true', 'false'),
@@ -130,7 +131,8 @@ def _replace_root_uuid(path, root_uuid):
 
 def _replace_boot_line(path, boot_mode, is_whole_disk_image,
                        trusted_boot=False, iscsi_boot=False,
-                       ramdisk_boot=False, ipxe_enabled=False):
+                       ramdisk_boot=False, ipxe_enabled=False,
+                       anaconda_boot=False):
     if is_whole_disk_image:
         boot_disk_type = 'boot_whole_disk'
     elif trusted_boot:
@@ -139,6 +141,8 @@ def _replace_boot_line(path, boot_mode, is_whole_disk_image,
         boot_disk_type = 'boot_iscsi'
     elif ramdisk_boot:
         boot_disk_type = 'boot_ramdisk'
+    elif anaconda_boot:
+        boot_disk_type = 'boot_anaconda'
     else:
         boot_disk_type = 'boot_partition'
 
@@ -162,7 +166,7 @@ def _replace_disk_identifier(path, disk_identifier):
 def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
                       is_whole_disk_image, trusted_boot=False,
                       iscsi_boot=False, ramdisk_boot=False,
-                      ipxe_enabled=False):
+                      ipxe_enabled=False, anaconda_boot=False):
     """Switch a pxe config from deployment mode to service mode.
 
     :param path: path to the pxe config file in tftpboot.
@@ -177,15 +181,17 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
     :param ramdisk_boot: if the boot is to be to a ramdisk configuration.
     :param ipxe_enabled: A default False boolean value to tell the method
                          if the caller is using iPXE.
+    :param anaconda_boot: if the boot is to be to an anaconda configuration.
     """
-    if not ramdisk_boot and root_uuid_or_disk_id is not None:
+    if (not (ramdisk_boot or anaconda_boot)
+            and root_uuid_or_disk_id is not None):
         if not is_whole_disk_image:
             _replace_root_uuid(path, root_uuid_or_disk_id)
         else:
             _replace_disk_identifier(path, root_uuid_or_disk_id)
 
     _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot,
-                       iscsi_boot, ramdisk_boot, ipxe_enabled)
+                       iscsi_boot, ramdisk_boot, ipxe_enabled, anaconda_boot)
 
 
 def check_for_missing_params(info_dict, error_msg, param_prefix=''):
@@ -307,10 +313,20 @@ def agent_add_clean_params(task):
     zeroize = CONF.deploy.shred_final_overwrite_with_zeros
     info['agent_erase_devices_zeroize'] = zeroize
     erase_fallback = CONF.deploy.continue_if_disk_secure_erase_fails
+    info['agent_continue_if_secure_erase_failed'] = erase_fallback
+    # NOTE(janders) ``agent_continue_if_ata_erase_failed`` is deprecated and
+    # will be removed in the "Y" cycle. The replacement option
+    # ``agent_continue_if_secure_erase_failed`` is used to control shred
+    #  fallback for both ATA Secure Erase and NVMe Secure Erase.
+    # The ``agent_continue_if_ata_erase_failed`` line can
+    # be deleted along with this comment when support for it is fully removed.
     info['agent_continue_if_ata_erase_failed'] = erase_fallback
+    nvme_secure_erase = CONF.deploy.enable_nvme_secure_erase
+    info['agent_enable_nvme_secure_erase'] = nvme_secure_erase
     secure_erase = CONF.deploy.enable_ata_secure_erase
     info['agent_enable_ata_secure_erase'] = secure_erase
     info['disk_erasure_concurrency'] = CONF.deploy.disk_erasure_concurrency
+    info['agent_erase_skip_read_only'] = CONF.deploy.erase_skip_read_only
 
     task.node.driver_internal_info = info
     task.node.save()
@@ -354,7 +370,13 @@ def get_disk_label(node):
     :returns: the disk label or None if no disk label was specified.
     """
     capabilities = utils.parse_instance_info_capabilities(node)
-    return capabilities.get('disk_label')
+    label = capabilities.get('disk_label')
+    # NOTE(TheJulia): If the node is UEFI based, we should likely just default
+    # the table type to gpt as otherwise we rely upon the user to supply the
+    # right information, and for UEFI mode bios may work, but is wrong.
+    if label is None and boot_mode_utils.get_boot_mode(node) == 'uefi':
+        label = 'gpt'
+    return label
 
 
 def get_pxe_boot_file(node):
@@ -491,29 +513,17 @@ def validate_capabilities(node):
                  'value': value, 'valid_values': ', '.join(valid_values)})
 
 
-def validate_image_properties(ctx, deploy_info, properties):
-    """Validate the image.
-
-    For Glance images it checks that the image exists in Glance and its
-    properties or deployment info contain the properties passed. If it's not a
-    Glance image, it checks that deployment info contains needed properties.
+def get_image_properties(ctx, image_href):
+    """Get properties of the image.
 
     :param ctx: security context
-    :param deploy_info: the deploy_info to be validated
-    :param properties: the list of image meta-properties to be validated.
-    :raises: InvalidParameterValue if:
-        * connection to glance failed;
-        * authorization for accessing image failed;
-        * HEAD request to image URL failed or returned response code != 200;
-        * HEAD request response does not contain Content-Length header;
-        * the protocol specified in image URL is not supported.
-    :raises: MissingParameterValue if the image doesn't contain
-        the mentioned properties.
+    :param image_href: reference to the image
+    :return: properties as a dictionary
+    :raises: InvalidParameterValue if the image cannot be accessed
     """
-    image_href = deploy_info['image_source']
     try:
         img_service = image_service.get_image_service(image_href, context=ctx)
-        image_props = img_service.show(image_href)['properties']
+        return img_service.show(image_href)['properties']
     except (exception.GlanceConnectionFailed,
             exception.ImageNotAuthorized,
             exception.Invalid):
@@ -526,6 +536,53 @@ def validate_image_properties(ctx, deploy_info, properties):
     except exception.ImageRefValidationFailed as e:
         raise exception.InvalidParameterValue(err=e)
 
+
+def validate_image_properties(task, deploy_info):
+    """Validate the image.
+
+    For Glance images it checks that the image exists in Glance and its
+    properties or deployment info contain the properties passed. If it's not a
+    Glance image, it checks that deployment info contains needed properties.
+
+    :param task: TaskManager instance with a valid node
+    :param deploy_info: the deploy_info to be validated
+    :raises: InvalidParameterValue if:
+        * connection to glance failed;
+        * authorization for accessing image failed;
+        * HEAD request to image URL failed or returned response code != 200;
+        * HEAD request response does not contain Content-Length header;
+        * the protocol specified in image URL is not supported.
+    :raises: MissingParameterValue if the image doesn't contain
+        the mentioned properties.
+    """
+    image_href = deploy_info.get('image_source')
+    boot_iso = deploy_info.get('boot_iso')
+    if image_href and boot_iso:
+        raise exception.InvalidParameterValue(_(
+            "An 'image_source' and 'boot_iso' parameter may not be "
+            "specified at the same time."))
+    if not image_href:
+        image_href = boot_iso
+
+    properties = []
+    if boot_iso or task.node.driver_internal_info.get('is_whole_disk_image'):
+        # No image properties are required in this case
+        return
+
+    if service_utils.is_glance_image(image_href):
+        properties = ['kernel_id', 'ramdisk_id']
+        boot_option = get_boot_option(task.node)
+        if boot_option == 'kickstart':
+            properties.append('stage2_id')
+    else:
+        properties = ['kernel', 'ramdisk']
+
+    if image_href:
+        image_props = get_image_properties(task.context, image_href)
+    else:
+        # Ramdisk deploy, no image_source is present
+        image_props = []
+
     missing_props = []
     for prop in properties:
         if not (deploy_info.get(prop) or image_props.get(prop)):
@@ -557,11 +614,35 @@ def get_boot_option(node):
     # NOTE(TheJulia): Software raid always implies local deployment
     if is_software_raid(node):
         return 'local'
+    if is_anaconda_deploy(node):
+        return 'kickstart'
+    if is_ramdisk_deploy(node):
+        return 'ramdisk'
     capabilities = utils.parse_instance_info_capabilities(node)
     return capabilities.get('boot_option',
                             CONF.deploy.default_boot_option).lower()
 
 
+# FIXME(dtantsur): relying on deploy interface name is an anti-pattern.
+# Refactor the code so that the deploy interface itself provides the only boot
+# option it supports.
+
+def is_ramdisk_deploy(node):
+    return node.deploy_interface == 'ramdisk'
+
+
+def is_anaconda_deploy(node):
+    """Determine if Anaconda deploy interface is in use for the deployment.
+
+    :param node: A single Node.
+    :returns: A boolean value of True when Anaconda deploy interface is in use
+              otherwise False
+    """
+    if node.deploy_interface == 'anaconda':
+        return True
+    return False
+
+
 def is_software_raid(node):
     """Determine if software raid is in use for the deployment.
 
@@ -579,6 +660,9 @@ def is_software_raid(node):
     return software_raid
 
 
+IPA_URL_PARAM_NAME = 'ipa-api-url'
+
+
 def build_agent_options(node):
     """Build the options to be passed to the agent ramdisk.
 
@@ -587,7 +671,7 @@ def build_agent_options(node):
         agent ramdisk.
     """
     agent_config_opts = {
-        'ipa-api-url': get_ironic_api_url(),
+        IPA_URL_PARAM_NAME: get_ironic_api_url(),
     }
     return agent_config_opts
 
@@ -640,11 +724,13 @@ def prepare_inband_cleaning(task, manage_boot=True):
         ramdisk_opts = build_agent_options(task.node)
         task.driver.boot.prepare_ramdisk(task, ramdisk_opts)
 
+    # NOTE(dtantsur): calling prepare_ramdisk may power off the node, so we
+    # need to check fast-track again and reboot if needed.
+    fast_track = manager_utils.is_fast_track(task)
     if not fast_track:
         manager_utils.node_power_action(task, states.REBOOT)
-
-    # Tell the conductor we are waiting for the agent to boot.
-    return states.CLEANWAIT
+        # Tell the conductor we are waiting for the agent to boot.
+        return states.CLEANWAIT
 
 
 def tear_down_inband_cleaning(task, manage_boot=True):
@@ -697,14 +783,34 @@ def get_image_instance_info(node):
         instance_info. Also raises same exception if kernel/ramdisk is
         missing in instance_info for non-glance images.
     """
+    # TODO(TheJulia): We seem to have a lack of direct unit testing of this
+    # method, but that is likely okay. If memory serves we test this at
+    # a few different levels. That being said, it would be good for some
+    # more explicit unit testing to exist.
     info = {}
-    info['image_source'] = node.instance_info.get('image_source')
 
-    is_whole_disk_image = node.driver_internal_info.get('is_whole_disk_image')
-    if not is_whole_disk_image:
-        if not service_utils.is_glance_image(info['image_source']):
+    boot_iso = node.instance_info.get('boot_iso')
+    image_source = node.instance_info.get('image_source')
+    if boot_iso:
+        if image_source:
+            raise exception.InvalidParameterValue(_(
+                "An 'image_source' and 'boot_iso' parameter may not be "
+                "specified at the same time."))
+        info['boot_iso'] = boot_iso
+    else:
+        if get_boot_option(node) == 'ramdisk':
+            # Ramdisk deploy does not require an image
             info['kernel'] = node.instance_info.get('kernel')
             info['ramdisk'] = node.instance_info.get('ramdisk')
+        else:
+            info['image_source'] = image_source
+
+            is_whole_disk_image = node.driver_internal_info.get(
+                'is_whole_disk_image')
+            if (not is_whole_disk_image
+                    and not service_utils.is_glance_image(image_source)):
+                info['kernel'] = node.instance_info.get('kernel')
+                info['ramdisk'] = node.instance_info.get('ramdisk')
 
     error_msg = (_("Cannot validate image information for node %s because one "
                    "or more parameters are missing from its instance_info and "
@@ -949,6 +1055,112 @@ def destroy_http_instance_images(node):
     destroy_images(node.uuid)
 
 
+def _validate_image_url(node, url, secret=False):
+    """Validates image URL through the HEAD request.
+
+    :param url: URL to be validated
+    :param secret: if URL is secret (e.g. swift temp url),
+        it will not be shown in logs.
+    """
+    try:
+        image_service.HttpImageService().validate_href(url, secret)
+    except exception.ImageRefValidationFailed as e:
+        with excutils.save_and_reraise_exception():
+            LOG.error("The specified URL is not a valid HTTP(S) URL or is "
+                      "not reachable for node %(node)s. Error: %(msg)s",
+                      {'node': node.uuid, 'msg': e})
+
+
+def _cache_and_convert_image(task, instance_info, image_info=None):
+    """Cache an image locally and covert it to RAW if needed."""
+    # Ironic cache and serve images from httpboot server
+    force_raw = direct_deploy_should_convert_raw_image(task.node)
+    _, image_path = cache_instance_image(task.context, task.node,
+                                         force_raw=force_raw)
+    if force_raw or image_info is None:
+        if force_raw:
+            instance_info['image_disk_format'] = 'raw'
+        else:
+            LOG.debug('Detecting image format for the locally cached image '
+                      '%(image)s for node %(node)s',
+                      {'image': image_path, 'node': task.node.uuid})
+            instance_info['image_disk_format'] = \
+                images.get_source_format(instance_info['image_source'],
+                                         image_path)
+
+        # Standard behavior is for image_checksum to be MD5,
+        # so if the hash algorithm is None, then we will use
+        # sha256.
+        if image_info is None:
+            os_hash_algo = instance_info.get('image_os_hash_algo')
+        else:
+            os_hash_algo = image_info.get('os_hash_algo')
+
+        if not os_hash_algo or os_hash_algo == 'md5':
+            LOG.debug("Checksum algorithm for image %(image)s for node "
+                      "%(node)s is set to '%(algo)s', changing to 'sha256'",
+                      {'algo': os_hash_algo, 'node': task.node.uuid,
+                       'image': image_path})
+            os_hash_algo = 'sha256'
+
+        LOG.debug('Recalculating checksum for image %(image)s for node '
+                  '%(node)s due to image conversion',
+                  {'image': image_path, 'node': task.node.uuid})
+        instance_info['image_checksum'] = None
+        hash_value = compute_image_checksum(image_path, os_hash_algo)
+        instance_info['image_os_hash_algo'] = os_hash_algo
+        instance_info['image_os_hash_value'] = hash_value
+    else:
+        instance_info['image_checksum'] = image_info['checksum']
+        instance_info['image_disk_format'] = image_info['disk_format']
+        instance_info['image_os_hash_algo'] = image_info[
+            'os_hash_algo']
+        instance_info['image_os_hash_value'] = image_info[
+            'os_hash_value']
+
+    # Create symlink and update image url
+    symlink_dir = _get_http_image_symlink_dir_path()
+    fileutils.ensure_tree(symlink_dir)
+    symlink_path = _get_http_image_symlink_file_path(task.node.uuid)
+    file_extension = None
+    if get_boot_option(task.node) == 'kickstart':
+        # 'liveimg --url' kickstart command uses the file extension to
+        # identify the OS image type. Without a valid file extension it will
+        # assume the disk image is a partition image and try to 'mount' it on
+        # the ramdisk. See 'liveimg' command for more details
+        # https://pykickstart.readthedocs.io/en/latest/kickstart-docs.html
+        valid_file_extensions = ['.img', '.tar', '.tbz', '.tgz', '.txz',
+                                 '.tar.gz', '.tar.bz2', '.tar.xz']
+        if image_info and 'disk_file_extension' in image_info['properties']:
+            ext = image_info['properties']['disk_file_extension']
+            file_extension = ext if ext in valid_file_extensions else None
+        if file_extension:
+            symlink_path = symlink_path + file_extension
+        else:
+            LOG.warning("The 'disk_file_extension' property was not set on "
+                        "the glance image or not a valid extension so the "
+                        "anaconda installer will try to 'mount' the OS image.")
+    utils.create_link_without_raise(image_path, symlink_path)
+
+    base_url = CONF.deploy.http_url
+    if base_url.endswith('/'):
+        base_url = base_url[:-1]
+    http_image_url = '/'.join(
+        [base_url, CONF.deploy.http_image_subdir,
+         task.node.uuid])
+    if file_extension:
+        http_image_url = http_image_url + file_extension
+    _validate_image_url(task.node, http_image_url, secret=False)
+    instance_info['image_url'] = http_image_url
+
+
+def get_image_download_source(node):
+    """Get the effective value of image_download_source for the node."""
+    return (node.instance_info.get('image_download_source')
+            or node.driver_info.get('image_download_source')
+            or CONF.agent.image_download_source)
+
+
 @METRICS.timer('build_instance_info_for_deploy')
 def build_instance_info_for_deploy(task):
     """Build instance_info necessary for deploying to a node.
@@ -959,85 +1171,27 @@ def build_instance_info_for_deploy(task):
     :raises: exception.ImageRefValidationFailed if image_source is not
         Glance href and is not HTTP(S) URL.
     """
-    def validate_image_url(url, secret=False):
-        """Validates image URL through the HEAD request.
-
-        :param url: URL to be validated
-        :param secret: if URL is secret (e.g. swift temp url),
-            it will not be shown in logs.
-        """
-        try:
-            image_service.HttpImageService().validate_href(url, secret)
-        except exception.ImageRefValidationFailed as e:
-            with excutils.save_and_reraise_exception():
-                LOG.error("Agent deploy supports only HTTP(S) URLs as "
-                          "instance_info['image_source'] or swift "
-                          "temporary URL. Either the specified URL is not "
-                          "a valid HTTP(S) URL or is not reachable "
-                          "for node %(node)s. Error: %(msg)s",
-                          {'node': node.uuid, 'msg': e})
     node = task.node
     instance_info = node.instance_info
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
     image_source = instance_info['image_source']
+    image_download_source = get_image_download_source(node)
 
     if service_utils.is_glance_image(image_source):
         glance = image_service.GlanceImageService(context=task.context)
         image_info = glance.show(image_source)
         LOG.debug('Got image info: %(info)s for node %(node)s.',
                   {'info': image_info, 'node': node.uuid})
-        if CONF.agent.image_download_source == 'swift':
+        if image_download_source == 'swift':
             swift_temp_url = glance.swift_temp_url(image_info)
-            validate_image_url(swift_temp_url, secret=True)
+            _validate_image_url(node, swift_temp_url, secret=True)
             instance_info['image_url'] = swift_temp_url
             instance_info['image_checksum'] = image_info['checksum']
             instance_info['image_disk_format'] = image_info['disk_format']
             instance_info['image_os_hash_algo'] = image_info['os_hash_algo']
             instance_info['image_os_hash_value'] = image_info['os_hash_value']
         else:
-            # Ironic cache and serve images from httpboot server
-            force_raw = direct_deploy_should_convert_raw_image(node)
-            _, image_path = cache_instance_image(task.context, node,
-                                                 force_raw=force_raw)
-            if force_raw:
-                instance_info['image_disk_format'] = 'raw'
-                # Standard behavior is for image_checksum to be MD5,
-                # so if the hash algorithm is None, then we will use
-                # sha256.
-                os_hash_algo = image_info.get('os_hash_algo')
-                if os_hash_algo == 'md5':
-                    LOG.debug('Checksum calculation for image %(image)s is '
-                              'set to \'%(algo)s\', changing to \'sha256\'',
-                              {'algo': os_hash_algo,
-                               'image': image_path})
-                    os_hash_algo = 'sha256'
-                LOG.debug('Recalculating checksum for image %(image)s due to '
-                          'image conversion.', {'image': image_path})
-                instance_info['image_checksum'] = None
-                hash_value = compute_image_checksum(image_path, os_hash_algo)
-                instance_info['image_os_hash_algo'] = os_hash_algo
-                instance_info['image_os_hash_value'] = hash_value
-            else:
-                instance_info['image_checksum'] = image_info['checksum']
-                instance_info['image_disk_format'] = image_info['disk_format']
-                instance_info['image_os_hash_algo'] = image_info[
-                    'os_hash_algo']
-                instance_info['image_os_hash_value'] = image_info[
-                    'os_hash_value']
-
-            # Create symlink and update image url
-            symlink_dir = _get_http_image_symlink_dir_path()
-            fileutils.ensure_tree(symlink_dir)
-            symlink_path = _get_http_image_symlink_file_path(node.uuid)
-            utils.create_link_without_raise(image_path, symlink_path)
-            base_url = CONF.deploy.http_url
-            if base_url.endswith('/'):
-                base_url = base_url[:-1]
-            http_image_url = '/'.join(
-                [base_url, CONF.deploy.http_image_subdir,
-                 node.uuid])
-            validate_image_url(http_image_url, secret=True)
-            instance_info['image_url'] = http_image_url
+            _cache_and_convert_image(task, instance_info, image_info)
 
         instance_info['image_container_format'] = (
             image_info['container_format'])
@@ -1047,8 +1201,11 @@ def build_instance_info_for_deploy(task):
         if not iwdi:
             instance_info['kernel'] = image_info['properties']['kernel_id']
             instance_info['ramdisk'] = image_info['properties']['ramdisk_id']
+    elif (image_source.startswith('file://')
+          or image_download_source == 'local'):
+        _cache_and_convert_image(task, instance_info)
     else:
-        validate_image_url(image_source)
+        _validate_image_url(node, image_source)
         instance_info['image_url'] = image_source
 
     if not iwdi:
