@@ -19,10 +19,13 @@ import tempfile
 from unittest import mock
 
 from oslo_utils import importutils
+from oslo_utils import uuidutils
 
 from ironic.common import images
+from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import task_manager
+from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
@@ -115,7 +118,7 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
 
         self.assertEqual(
-            'http://localhost/redfish/boot.iso?filename=file.iso', url)
+            'http://localhost/redfish/boot.iso', url)
 
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
@@ -137,7 +140,7 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
 
         self.assertEqual(
-            'http://non-local.host/redfish/boot.iso?filename=file.iso', url)
+            'http://non-local.host/redfish/boot.iso', url)
 
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
@@ -159,7 +162,7 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
 
         self.assertEqual(
-            'http://localhost/redfish/boot.iso?filename=file.iso', url)
+            'http://localhost/redfish/boot.iso', url)
 
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
 
@@ -239,7 +242,8 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                     enabled_inspect_interfaces=['redfish'],
                     enabled_bios_interfaces=['redfish'])
         self.node = obj_utils.create_test_node(
-            self.context, driver='redfish', driver_info=INFO_DICT)
+            self.context, driver='redfish', driver_info=INFO_DICT,
+            provision_state=states.DEPLOYING)
 
     @mock.patch.object(image_utils.ImageHandler, 'unpublish_image',
                        autospec=True)
@@ -577,6 +581,79 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
                 base_iso='/path/to/baseiso', inject_files=None)
 
+    @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_kernel_params_for_ramdisk(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+
+            task.node.instance_info['ramdisk_kernel_arguments'] = kernel_params
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid)
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='bios', esp_image_href=None,
+                kernel_params="root=/dev/ram0 text " + kernel_params,
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                base_iso=None, inject_files=None)
+
+    @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_kernel_params_for_ramdisk_boot_iso(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+
+            task.node.instance_info['ramdisk_kernel_arguments'] = kernel_params
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid,
+                base_iso='/path/to/baseiso')
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='bios', esp_image_href=None,
+                # No custom parameters with a boot ISO present
+                kernel_params=None,
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                base_iso='/path/to/baseiso', inject_files=None)
+
+    @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_kernel_params_for_ramdisk_cleaning(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+
+            task.node.driver_info['kernel_append_params'] = kernel_params
+            task.node.provision_state = states.CLEANING
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid,
+                base_iso='/path/to/baseiso')
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='bios', esp_image_href=None,
+                kernel_params=kernel_params,
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                base_iso='/path/to/baseiso', inject_files=None)
+
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
@@ -610,6 +687,17 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 base_iso=base_image_url)
             self.assertEqual(url, base_image_url)
 
+    @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
+    def test__prepare_iso_image_bootable_iso_with_instance_info(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = 'http://bearmetal.net/boot.iso'
+            task.node.instance_info['ramdisk_image_download_source'] = 'http'
+            url = image_utils._prepare_iso_image(
+                task, None, None, bootloader_href=None, root_uuid=None,
+                base_iso=base_image_url)
+            self.assertEqual(url, base_image_url)
+
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
@@ -630,11 +718,43 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 kernel_params='nofb nomodeset vga=normal', boot_mode='bios',
                 base_iso='/path/to/baseiso', inject_files=None)
 
+    @mock.patch.object(images, 'get_temp_url_for_glance_image',
+                       autospec=True)
+    def test__prepare_iso_image_bootable_iso_from_swift(self, mock_temp_url):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = uuidutils.generate_uuid()
+            self.config(ramdisk_image_download_source='swift', group='deploy')
+            url = image_utils._prepare_iso_image(
+                task, None, None, bootloader_href=None, root_uuid=None,
+                base_iso=base_image_url)
+            self.assertEqual(mock_temp_url.return_value, url)
+            mock_temp_url.assert_called_once_with(task.context, base_image_url)
+
+    def test__prepare_iso_image_bootable_iso_swift_noop(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = 'http://bearmetal.net/boot.iso'
+            self.config(ramdisk_image_download_source='swift', group='deploy')
+            url = image_utils._prepare_iso_image(
+                task, None, None, bootloader_href=None, root_uuid=None,
+                base_iso=base_image_url)
+            self.assertEqual(url, base_image_url)
+
+    def test__prepare_iso_image_bootable_iso_swift_schema(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = 'swift://bearmetal.net/boot.iso'
+            url = image_utils._prepare_iso_image(
+                task, None, None, bootloader_href=None, root_uuid=None,
+                base_iso=base_image_url)
+            self.assertEqual(url, base_image_url)
+
     def test__find_param(self):
         param_dict = {
-            'ilo_deploy_kernel': 'kernel',
-            'ilo_deploy_ramdisk': 'ramdisk',
-            'ilo_bootloader': 'bootloader'
+            'deploy_kernel': 'kernel',
+            'deploy_ramdisk': 'ramdisk',
+            'bootloader': 'bootloader'
         }
         param_str = "deploy_kernel"
         expected = "kernel"
@@ -644,8 +764,8 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
     def test__find_param_not_found(self):
         param_dict = {
-            'ilo_deploy_ramdisk': 'ramdisk',
-            'ilo_bootloader': 'bootloader'
+            'deploy_ramdisk': 'ramdisk',
+            'bootloader': 'bootloader'
         }
         param_str = "deploy_kernel"
         expected = None
@@ -658,9 +778,9 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                                   shared=True) as task:
 
             d_info = {
-                'ilo_deploy_kernel': 'kernel',
-                'ilo_deploy_ramdisk': 'ramdisk',
-                'ilo_bootloader': 'bootloader'
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk',
+                'bootloader': 'bootloader'
             }
             task.node.driver_info.update(d_info)
 
@@ -715,8 +835,8 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                                   shared=True) as task:
 
             d_info = {
-                'ilo_deploy_kernel': 'kernel',
-                'ilo_deploy_ramdisk': 'ramdisk'
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk'
             }
             task.node.driver_info.update(d_info)
 
@@ -753,9 +873,9 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                                   shared=True) as task:
 
             d_info = {
-                'ilo_deploy_kernel': 'kernel',
-                'ilo_deploy_ramdisk': 'ramdisk',
-                'ilo_bootloader': 'bootloader'
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk',
+                'bootloader': 'bootloader'
             }
             task.node.driver_info.update(d_info)
 
@@ -781,9 +901,9 @@ cafile = /etc/ironic-python-agent/ironic.crt
                                   shared=True) as task:
 
             d_info = {
-                'ilo_deploy_kernel': 'kernel',
-                'ilo_deploy_ramdisk': 'ramdisk',
-                'ilo_bootloader': 'bootloader'
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk',
+                'bootloader': 'bootloader'
             }
             task.node.driver_info.update(d_info)
 
@@ -804,9 +924,9 @@ cafile = /etc/ironic-python-agent/ironic.crt
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             d_info = {
-                'ilo_deploy_kernel': 'kernel',
-                'ilo_deploy_ramdisk': 'ramdisk',
-                'ilo_bootloader': 'bootloader'
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk',
+                'bootloader': 'bootloader'
             }
             task.node.driver_info.update(d_info)
 
