@@ -236,18 +236,17 @@ class PXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.UnsupportedDriverExtension,
                               task.driver.boot.validate_inspection, task)
 
-    @mock.patch.object(deploy_utils, 'validate_image_properties',
-                       autospec=True)
-    def test_validate_kickstart_has_squashfs_id(self, mock_validate_img):
-        node = self.node
-        node.deploy_interface = 'anaconda'
-        node.save()
+    @mock.patch.object(image_service.GlanceImageService, 'show', autospec=True)
+    def test_validate_kickstart_missing_stage2_id(self, mock_glance):
+        mock_glance.return_value = {'properties': {'kernel_id': 'fake-kernel',
+                                                   'ramdisk_id': 'fake-initr'}}
+        self.node.deploy_interface = 'anaconda'
+        self.node.save()
         self.config(http_url='http://fake_url', group='deploy')
-        with task_manager.acquire(self.context, node.uuid) as task:
-            task.driver.boot.validate(task)
-            mock_validate_img.assert_called_once_with(
-                mock.ANY, mock.ANY, ['kernel_id', 'ramdisk_id', 'squashfs_id']
-            )
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertRaisesRegex(exception.MissingParameterValue,
+                                   'stage2_id',
+                                   task.driver.boot.validate, task)
 
     def test_validate_kickstart_fail_http_url_not_set(self):
         node = self.node
@@ -854,7 +853,8 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
                             'default_%s_interface' % iface: impl}
             self.config(**config_kwarg)
         self.config(enabled_hardware_types=['fake-hardware'])
-        instance_info = INST_INFO_DICT
+        instance_info = {'kernel': 'kernelUUID',
+                         'ramdisk': 'ramdiskUUID'}
         self.node = obj_utils.create_test_node(
             self.context,
             driver='fake-hardware',
@@ -964,8 +964,16 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
                        autospec=True)
     def test_validate(self, mock_validate_img):
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info['capabilities'] = {
-                'boot_option': 'netboot'}
+            task.driver.deploy.validate(task)
+        self.assertTrue(mock_validate_img.called)
+
+    @mock.patch.object(deploy_utils, 'validate_image_properties',
+                       autospec=True)
+    def test_validate_with_boot_iso(self, mock_validate_img):
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.instance_info = {
+                'boot_iso': 'isoUUID'
+            }
             task.driver.deploy.validate(task)
         self.assertTrue(mock_validate_img.called)
 
@@ -1033,6 +1041,58 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
             self.assertIsNone(task.driver.deploy.deploy(task))
             self.assertTrue(mock_warning.called)
 
+    @mock.patch.object(agent_base, 'get_steps', autospec=True)
+    def test_get_clean_steps(self, mock_get_steps):
+        # Test getting clean steps
+        mock_steps = [{'priority': 10, 'interface': 'deploy',
+                       'step': 'erase_devices'}]
+        mock_get_steps.return_value = mock_steps
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            steps = task.driver.deploy.get_clean_steps(task)
+            mock_get_steps.assert_called_once_with(
+                task, 'clean', interface='deploy',
+                override_priorities={'erase_devices': None,
+                                     'erase_devices_metadata': None})
+        self.assertEqual(mock_steps, steps)
+
+    def test_get_deploy_steps(self):
+        # Only the default deploy step exists in the ramdisk deploy
+        expected = [{'argsinfo': None, 'interface': 'deploy', 'priority': 100,
+                     'step': 'deploy'}]
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            steps = task.driver.deploy.get_deploy_steps(task)
+            self.assertEqual(expected, steps)
+
+    @mock.patch.object(agent_base, 'execute_step', autospec=True)
+    def test_execute_clean_step(self, mock_execute_step):
+        step = {
+            'priority': 10,
+            'interface': 'deploy',
+            'step': 'erase_devices',
+            'reboot_requested': False
+        }
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            result = task.driver.deploy.execute_clean_step(task, step)
+            self.assertIs(result, mock_execute_step.return_value)
+            mock_execute_step.assert_called_once_with(task, step, 'clean')
+
+    @mock.patch.object(deploy_utils, 'prepare_inband_cleaning', autospec=True)
+    def test_prepare_cleaning(self, prepare_inband_cleaning_mock):
+        prepare_inband_cleaning_mock.return_value = states.CLEANWAIT
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertEqual(
+                states.CLEANWAIT, task.driver.deploy.prepare_cleaning(task))
+            prepare_inband_cleaning_mock.assert_called_once_with(
+                task, manage_boot=True)
+
+    @mock.patch.object(deploy_utils, 'tear_down_inband_cleaning',
+                       autospec=True)
+    def test_tear_down_cleaning(self, tear_down_cleaning_mock):
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.driver.deploy.tear_down_cleaning(task)
+            tear_down_cleaning_mock.assert_called_once_with(
+                task, manage_boot=True)
+
 
 class PXEAnacondaDeployTestCase(db_base.DbTestCase):
 
@@ -1087,7 +1147,9 @@ class PXEAnacondaDeployTestCase(db_base.DbTestCase):
                       'ks_cfg': ('', '/path/to/ks_cfg')}
         mock_image_info.return_value = image_info
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertIsNone(task.driver.deploy.deploy(task))
+            self.assertEqual(
+                states.DEPLOYWAIT, task.driver.deploy.deploy(task)
+            )
             mock_image_info.assert_called_once_with(task, ipxe_enabled=False)
             mock_cache.assert_called_once_with(
                 task, image_info, ipxe_enabled=False)
@@ -1126,6 +1188,8 @@ class PXEAnacondaDeployTestCase(db_base.DbTestCase):
                       'ks_template': ('', '/path/to/ks_template'),
                       'ks_cfg': ('', '/path/to/ks_cfg')}
         mock_image_info.return_value = image_info
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.driver.deploy.reboot_to_instance(task)
             mock_set_boot_dev.assert_called_once_with(task, boot_devices.DISK)
@@ -1188,6 +1252,17 @@ class PXEAnacondaDeployTestCase(db_base.DbTestCase):
                 task.node.driver_internal_info['agent_status'])
             self.assertTrue(mock_reboot_to_instance.called)
 
+    @mock.patch.object(deploy_utils, 'prepare_inband_cleaning', autospec=True)
+    def test_prepare_cleaning(self, prepare_inband_cleaning_mock):
+        prepare_inband_cleaning_mock.return_value = states.CLEANWAIT
+        self.node.provision_state = states.CLEANING
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertEqual(
+                states.CLEANWAIT, self.deploy.prepare_cleaning(task))
+            prepare_inband_cleaning_mock.assert_called_once_with(
+                task, manage_boot=True)
+
 
 class PXEValidateRescueTestCase(db_base.DbTestCase):
 
